/* 
   io.c - handles the io of (the partial) secrets.

   Copyright (C) 2000 Stefan Karrmann

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

$Id: io.c,v 1.5 2000/03/05 15:47:24 sk Exp $
$Log: io.c,v $
Revision 1.5  2000/03/05 15:47:24  sk
io for maximal threshold unified

Revision 1.4  2000/02/26 18:22:12  sk
Changed for binary secrets

Revision 1.3  2000/02/23 21:19:58  sk
First working Version

*/

#include <limits.h>
#include <stdio.h>
#include <sys/types.h>
#include "sharesecret.h"

/*
#define MPZ_ASCII_BASE (36)
*/
#define MPZ_ASCII_BASE (10)
#define BITSOFCHAR

/* TODO  only ready mpz now */
/* initializes secret, callee must (mpz_)clear it */
void
readSecret (mpz_t secret, FILE * src)
{
  int ret;
  ulong pos;
  mpz_t new;

  mpz_init (secret);
  mpz_init (new);
  pos = 0;

  for (ret = getc (src); EOF != ret; ret = getc (src))
    {
      mpz_set_ui (new, (ulong) ret);
      mpz_mul_2exp (new, new, pos);
      mpz_ior (secret, secret, new);
      pos = pos + CHAR_BIT;
    }
  mpz_setbit (secret, pos);	/* preserve the length of the secret */
}

void
writeSecret (mpz_t secret, FILE * dst)
{
  int ret;
  char ch;
  
  while (1 == mpz_cmp_ui (secret, 1)) /* use the length preserving bit(byte) */
    {
      ch = (unsigned char) (mpz_get_ui (secret));
      ret = putc (ch, dst);
      if (EOF == ret)
	error (EXIT_FAILURE, 0, _("Cannot write secret file"));
      mpz_tdiv_q_2exp (secret, secret, CHAR_BIT);
    }
}

void
writePartPair (mpz_t x, mpz_t y, mpz_t modulus,
	   size_t min, size_t split, FILE * dst)
{
  fprintf (dst, "1:%d/%d\n", min, split);
  mpz_out_str (dst, MPZ_ASCII_BASE, modulus);
  fputc ('\n', dst);
  mpz_out_str (dst, MPZ_ASCII_BASE, x);
  fputc ('\n', dst);
  mpz_out_str (dst, MPZ_ASCII_BASE, y);
}

void
writePartSingle (mpz_t y, size_t split, FILE * dst)
{
  fprintf (dst, "1:%d/%d\n0\n0\n", split, split);
  mpz_out_str (dst, MPZ_ASCII_BASE, y);
}

/* initializes y in every case and x, modulus if min != split
   callee must (mpz_)clear it or them.
*/
void
readPart (mpz_t x, mpz_t y, mpz_t modulus,
	  size_t * min, size_t * split, FILE * src)
{
  fscanf (src, "1:%d/%d\n", min, split);
  mpz_init (y);
  if ((* min) != (* split))
    {
      mpz_init (modulus); mpz_init (x);
      mpz_inp_str (modulus, src, MPZ_ASCII_BASE);
      fgetc (src);
      mpz_inp_str (x, src, MPZ_ASCII_BASE);
      fgetc (src);
    }
  else
    fscanf (src, "0\n0\n");
  mpz_inp_str (y, src, MPZ_ASCII_BASE);
}


